@@ -9,12 +9,26 @@ from webob import UnicodeMultiDict
 
 import schemaish, validatish
 from formish import util
-from dottedish import dotted, unflatten, set
+from dottedish import dotted, unflatten, set as dottedish_set
 from formish import validation
 from formish import widgets
 from formish.renderer import _default_renderer
 
-NOARG = object()
+UNSET = object()
+
+
+def mroattrs(cls, attr):
+    """
+    Yield the values of class attributes that were changed according to the
+    class hierarchy starting with and including the given class, ordered by
+    specificity (i.e. deepest first).
+    """
+    seen = set()
+    for cls in cls.__mro__:
+        value = getattr(cls, attr, None)
+        if value and value not in seen:
+            yield value
+            seen.add(value)
 
 def container_factory(parent_key, item_key):
     if item_key.isdigit():
@@ -54,12 +68,13 @@ def _cssname(self):
 
 def _classes(self):
     """ Works out a list of classes that should be applied to the field """
-    classes = [
-        'field',
-        re.sub('[0-9\*]+','n',_cssname(self)),
-        'type-%s'%self.attr.type.lower(),
-        ]
-    classes.append('widget-%s'%self.widget.type.lower())
+    schema_types = mroattrs(self.attr.__class__,'type')
+    widget_types = mroattrs(self.widget.widget.__class__,'type')
+    classes = ['field',re.sub('[0-9\*]+','n',_cssname(self))]
+    for t in schema_types:
+        classes.append('type-%s'%t.lower())
+    for t in widget_types:
+        classes.append('widget-%s'%t.lower())
     if self.required:
         classes.append('required')
     if self.widget.css_class is not None:
@@ -269,6 +284,34 @@ class Field(object):
         vars = {'field':self}
         return fall_back_renderer(renderer, name, widget, vars)
     
+    def seqdelete(self):
+        widget_type, widget = self.widget.template.split('.')
+        """ creates a seq delete hook if this is an item in a updateable sequence """
+        parentkey = '.'.join(self.name.split('.')[:-1])
+        if not parentkey:
+            return ''
+        parent = self.form.get_field(parentkey)
+        if getattr(parent.widget,'addremove',False) == False:
+            return ''
+        renderer = self.form.renderer
+        name = 'field/seqdelete'
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
+    
+    def seqgrab(self):
+        widget_type, widget = self.widget.template.split('.')
+        """ creates a seq grab hook if this is an item in a updateable sequence """
+        parentkey = '.'.join(self.name.split('.')[:-1])
+        if not parentkey:
+            return ''
+        parent = self.form.get_field(parentkey)
+        if getattr(parent.widget,'sortable',False) == False:
+            return ''
+        renderer = self.form.renderer
+        name = 'field/seqgrab'
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
+    
     def inputs(self):
         """ returns the templated widget """
         widget_type, widget = self.widget.template.split('.')
@@ -407,7 +450,11 @@ class Collection(object):
         """ return the fields widget bound with extra params. """
         
         try:
-            widget_type = BoundWidget(self.form.get_item_data(starify(self.name),'widget'),self)
+            w = self.form.get_item_data(starify(self.name),'widget')
+            if not isinstance(w, BoundWidget):
+                widget_type = BoundWidget(self.form.get_item_data(starify(self.name),'widget'),self)
+            else:
+                widget_type = w
         except KeyError:
             if self.type == 'group':
                 widget_type = BoundWidget(widgets.StructureDefault(),self)
@@ -417,14 +464,21 @@ class Collection(object):
         return widget_type
 
 
-    def get_field(self, segments):
+    def get_field(self, name):
         """ recursively get dotted field names """
+        segments = name.split('.')
         for field in self.fields:
+            if segments[0] == '*':
+                b = field.bind('*',field.attr)
+                if len(segments) == 1:
+                    return b
+                else:
+                    return b.get_field('.'.join(segments[1:]))
             if field.name.split('.')[-1] == segments[0]:
-                if isinstance(field, Field):
+                if len(segments) == 1:
                     return field
                 else:
-                    return field.get_field(segments[1:])
+                    return field.get_field(''.join(segments[1:]))
 
 
     def __getitem__(self, key):
@@ -449,6 +503,15 @@ class Collection(object):
         """
         return CollectionFieldsWrapper(self)
 
+    @property
+    def allfields(self):
+        fields = []
+        for field in self.fields:
+            if hasattr(field,'allfields'):
+                fields.extend( field.allfields )
+            else:
+                fields.append(field)
+        return fields
 
     def bind(self, attr_name, attr):
         """ 
@@ -496,12 +559,42 @@ class Collection(object):
         name = '%s/label'%widget_type
         vars = {'field':self}
         return fall_back_renderer(renderer, name, widget, vars)
-         
-    def metadata(self):
-        """ returns the metadata """
+
+    def seqgrab(self):
+        widget_type, widget = self.widget.template.split('.')
+        """ creates a seq grab hook if this is an item in a updateable sequence """
+        parentkey = '.'.join(self.name.split('.')[:-1])
+        if not parentkey:
+            return ''
+        parent = self.form.get_field(parentkey)
+        if getattr(parent.widget,'sortable',False) == False:
+            return ''
+        renderer = self.form.renderer
+        name = 'field/seqgrab'
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
+
+    def seqdelete(self):
+        widget_type, widget = self.widget.template.split('.')
+        """ creates a seq delete hook if this is an item in a updateable sequence """
+        parentkey = '.'.join(self.name.split('.')[:-1])
+        if not parentkey:
+            return ''
+        parent = self.form.get_field(parentkey)
+        if getattr(parent.widget,'addremove',False) == False:
+            return ''
+
+        renderer = self.form.renderer
+        name = 'field/seqdelete'
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
+    
+    
+    def inputs(self):
+        """ returns the templated widget """
         widget_type, widget = self.widget.template.split('.')
         renderer = self.form.renderer
-        name = '%s/metadata'%widget_type
+        name = 'field/inputs'
         vars = {'field':self}
         return fall_back_renderer(renderer, name, widget, vars)
 
@@ -577,6 +670,14 @@ class Sequence(Collection):
     @property
     def template(self):
         return self.bind('*', self.attr.attr)
+         
+    def metadata(self):
+        """ returns the metadata """
+        widget_type, widget = self.widget.template.split('.')
+        renderer = self.form.renderer
+        name = '%s/metadata'%widget_type
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
             
 
 class BoundWidget(object):
@@ -586,8 +687,11 @@ class BoundWidget(object):
     
 
     def __init__(self, widget, field):
+        if hasattr(field.form,'empty'):
+            widget.empty = field.form.empty
         self.__dict__['widget'] = widget
         self.__dict__['field'] = field
+
      
 
     def __getattr__(self, name):
@@ -616,8 +720,14 @@ class FormFieldsWrapper(ObjectWrapper):
         self.form = form
         ObjectWrapper.__init__(self, form.structure.fields)
 
-    def __call__(self):
-        return self.form.renderer('/formish/form/fields.html', {'form':self.form})
+    def keys(self):
+        keys = []
+        for f in self.form.fields:
+            keys.append(f.name)
+        return keys
+
+    def __call__(self,fields=None):
+        return self.form.renderer('/formish/form/fields.html', {'form':self.form,'fields':fields})
     
 
 class Form(object):
@@ -628,17 +738,18 @@ class Form(object):
     render and validate data.
     """
 
-    SUPPORTED_METHODS = ['GET', 'POST']
+    SUPPORTED_METHODS = ['get', 'post']
 
     renderer = _default_renderer
 
-    _element_name = None
     _name = None
 
     _request_data = None
 
     def __init__(self, structure, name=None, defaults=None, errors=None,
-                 action_url=None, renderer=None, method='POST', add_default_action=True, include_charset=True):
+                 action_url=None, renderer=None, method='post',
+                 add_default_action=True, include_charset=True,
+                 empty=UNSET):
         """
         Create a new form instance
 
@@ -663,7 +774,7 @@ class Form(object):
         :arg method: Option method, default POST
         :type method: string
         """
-        if method.upper() not in self.SUPPORTED_METHODS:
+        if method.lower() not in self.SUPPORTED_METHODS:
             raise ValueError("method must be one of GET or POST")
         # allow a single schema items to be used on a form
         if not isinstance(structure, schemaish.Structure):
@@ -687,6 +798,8 @@ class Form(object):
         self.method = method
         self.widget = widgets.StructureDefault()
         self.include_charset = include_charset
+        if empty is not UNSET:
+            self.empty = empty
 
     def __repr__(self):
         attributes = []
@@ -700,18 +813,6 @@ class Form(object):
             attributes.append('action_url=%r'%self.action_url)
         return 'formish.Form(%s)'%( ', '.join(attributes) )
 
-    def _element_name_get(self):
-        """ Set the element name """
-        return self._element_name
-
-    def _element_name_set(self, name):
-        """ Get the element name or raise an error """
-        if self._name is not None:
-            raise Exception("Named forms cannot be used as elements.")
-        self._element_name = name
-
-    element_name = property(_element_name_get, _element_name_set)
-
     def add_action(self, name=None, value=None, callback=None):
         """ 
         Add an action callable to the form
@@ -773,7 +874,7 @@ class Form(object):
         """
         if self._request_data is not None:
             return dotted(self._request_data)
-        self._request_data = self.widget.to_request_data(self.structure, self._defaults)
+        self._request_data = dotted(self.widget.to_request_data(self.structure, self._defaults))
         return dotted(self._request_data)
 
 
@@ -784,7 +885,7 @@ class Form(object):
         :arg request_data: raw request data (e.g. request.POST)
         :type request_data: Dictionary (dotted or nested or dotted or MultiDict)
         """
-        self._request_data = request_data
+        self._request_data = dotted(request_data)
 
 
     request_data = property(_get_request_data, _set_request_data)
@@ -825,8 +926,8 @@ class Form(object):
 
         # Convert request data to a dottedish friendly representation
         request_data = _unflatten_request_data(request_data)
-        self._request_data = request_data
-        self._request_data = self.widget.pre_parse_incoming_request_data(self.structure,request_data)
+        self._request_data = dotted(request_data)
+        self._request_data = dotted(self.widget.pre_parse_incoming_request_data(self.structure,request_data))
 
     def _get_request(self):
         return self._request
@@ -896,19 +997,19 @@ class Form(object):
         allowed = ['title', 'widget', 'description','default']
         if name in allowed:
             if name == 'default' and '*' not in key:
-                set(self.defaults,key,value,container_factory=container_factory)
+                dottedish_set(self.defaults,key,value,container_factory=container_factory)
             else:
                 self.item_data.setdefault(key, {})[name] = value
         else:
             raise KeyError('Cannot set data onto this attribute')
 
 
-    def get_item_data(self, key, name, default=NOARG):
+    def get_item_data(self, key, name, default=UNSET):
         """
         Access item data associates with a field key and an attribute name
         (e.g. title, widget, description')
         """
-        if default is NOARG:
+        if default is UNSET:
             data = self.item_data.get(key, {})[name]
         else:
             data = self.item_data.get(key, {}).get(name, default)
@@ -942,6 +1043,26 @@ class Form(object):
         """
         return FormFieldsWrapper(self)
 
+    def _has_upload_fields(self):
+        for f in self.allfields:
+            if isinstance(f.attr, schemaish.File):
+                return True
+        return False
+
+    @property
+    def allfields(self):
+        """
+        Return a generator that yields all of the fields at the top level of
+        the form (e.g. if a field is a subsection or sequence, it will be up to
+        the application to iterate that field's fields.
+        """
+        fields = []
+        for field in self.fields:
+            if hasattr(field,'allfields'):
+                fields.extend( field.allfields )
+            else:
+                fields.append(field)
+        return fields
 
 
     def get_field(self, name):
@@ -950,13 +1071,20 @@ class Form(object):
 
         :arg name: Dotted name e.g. names.0.firstname
         """
+        # XXX GET FIELD NEEDS TO CACHE THE * FIELDS
         segments = name.split('.')
         for field in self.fields:
+            if segments[0] == '*':
+                b = field.bind('*',field.attr)
+                if len(segments) == 1:
+                    return b
+                else:
+                    return b.get_field('.'.join(segments[1:]))
             if field.name.split('.')[-1] == segments[0]:
                 if len(segments) == 1:
                     return field
                 else:
-                    return field.get_field(segments[1:])
+                    return field.get_field('.'.join(segments[1:]))
 
 
     def __call__(self):
