@@ -1267,6 +1267,23 @@ def form_UploadStructure(request):
     form['myStruct.a'].widget = formish.FileUpload(filestore=CachedTempFilestore())
     return form
 
+def form_NestedStructures(request):
+    """
+    Allow the emission of parts of a form schema using field names.
+    """
+    schema = schemaish.Structure()
+    schema.add( 'a', schemaish.String())
+    schema.add( 'b', schemaish.String())
+
+    sub_schema = schemaish.Structure()
+    sub_schema.add('x',schemaish.String())
+
+    schema.add('c', sub_schema)
+
+
+    form = formish.Form(schema, 'form')
+
+    return form
 
 
 #########################
@@ -1274,17 +1291,52 @@ def form_UploadStructure(request):
 #   Sequences
 #
 
-def form_SequenceOfStringsWithSequenceWidgetOptions(request):
+def form_SequenceOfStrings(request):
     """
     A sequence with some defaults
     """
     schema = schemaish.Structure()
     schema.add( 'myList', schemaish.Sequence( schemaish.String() ))
 
-    form = formish.Form(schema, 'form')
-    form['myList'].widget = formish.SequenceDefault(min_start_fields=1,min_empty_start_fields=0, batch_add_count=5)
+    form = formish.Form(schema,'formname')
     return form
 
+def functest_SequenceOfStrings(self):
+    sel = self.selenium
+    sel.open("/SequenceOfStrings")
+    sel.click_at("css=#formname-myList--field > a", "")
+    self.assertEqual("", sel.get_value("formname-myList-0"))
+    self.assertEqual("", sel.get_value("formname-myList-1"))
+    sel.click_at("css=#formname-myList--field > a", "")
+    self.assertEqual("", sel.get_value("formname-myList-2"))
+    sel.click_at("css=#formname-myList-0--field .remove", "")
+    self.assertEqual("", sel.get_value("formname-myList-0"))
+    self.assertEqual("", sel.get_value("formname-myList-1"))
+    self.assertEqual(False, sel.is_element_present("css=#formname-myList-2"))
+    
+def form_SequenceOfStringsWithoutFormName(request):
+    """
+    A sequence with some defaults
+    """
+    schema = schemaish.Structure()
+    schema.add( 'myList', schemaish.Sequence( schemaish.String() ))
+
+    form = formish.Form(schema)
+    return form
+
+def functest_SequenceOfStringsWithoutFormName(self):
+    sel = self.selenium
+    sel.open("/SequenceOfStringsWithoutFormName")
+    sel.click_at("css=#myList--field > a", "")
+    self.assertEqual("", sel.get_value("myList-0"))
+    self.assertEqual("", sel.get_value("myList-1"))
+    sel.click_at("css=#myList--field > a", "")
+    self.assertEqual("", sel.get_value("myList-2"))
+    sel.click_at("css=#myList-0--field .remove", "")
+    self.assertEqual("", sel.get_value("myList-0"))
+    self.assertEqual("", sel.get_value("myList-1"))
+    self.assertEqual(False, sel.is_element_present("css=#myList-2"))
+
 def form_SequenceOfSimpleStructures(request):
     """
     A structure witin a sequence, should be enhanced with javascript
@@ -1299,6 +1351,94 @@ def form_SequenceOfSimpleStructures(request):
     form = formish.Form(schema, 'form')
     form['myList'].widget = formish.SequenceDefault()
     return form
+
+def functest_SequenceOfSimpleStructures(self):
+    sel = self.selenium
+    sel.open("/SequenceOfSimpleStructures")
+    sel.type("form-myList-0-a", "form-myList-0-a")
+    self.assertEqual("form-myList-0-a", sel.get_value("form-myList-0-a"))
+    self.assertEqual("", sel.get_value("form-myList-0-b"))
+
+    sel.click_at("css=#form-myList--field > a", "")
+
+    sel.type("form-myList-1-a", "form-myList-1-a")
+    self.assertEqual("form-myList-1-a", sel.get_value("form-myList-1-a"))
+    self.assertEqual("", sel.get_value("form-myList-1-b"))
+
+    sel.click_at("css=#form-myList--field > a", "")
+    
+    sel.type("form-myList-2-a", "form-myList-2-a")
+    self.assertEqual("form-myList-2-a", sel.get_value("form-myList-2-a"))
+    self.assertEqual("", sel.get_value("form-myList-2-b"))
+
+    sel.mouse_down_at("css=#form-myList-1--field .remove", "")
+    sel.mouse_up('body')
+
+    self.assertEqual("form-myList-0-a", sel.get_value("form-myList-0-a"))
+    self.assertEqual("form-myList-2-a", sel.get_value("form-myList-1-a"))
+    self.assertEqual("", sel.get_value("form-myList-1-b"))
+    self.assertEqual(False, sel.is_element_present("css=#form-myList-2"))
+
+
+
+def form_SequenceOfSequences(request):
+    """
+    A structure witin a sequence, should be enhanced with javascript
+    """
+
+    schema = schemaish.Structure()
+    schema.add( 'myList', schemaish.Sequence( schemaish.Sequence( schemaish.String() ) ))
+
+    form = formish.Form(schema, 'form')
+    return form
+
+def functest_SequenceOfSequences(self):
+
+    sel = self.selenium
+    sel.open("/SequenceOfSequences")
+    sel.type("form-myList-0-0", "form-myList-0-0")
+    self.assertEqual("form-myList-0-0", sel.get_value("form-myList-0-0"))
+
+    sel.click_at("css=#form-myList-0--field > a", "")
+
+    sel.type("form-myList-0-1", "form-myList-0-1")
+    self.assertEqual("form-myList-0-0", sel.get_value("form-myList-0-0"))
+    self.assertEqual("form-myList-0-1", sel.get_value("form-myList-0-1"))
+
+    sel.click_at("css=#form-myList--field > a", "")
+    sel.click_at("css=#form-myList-1--field > a", "")
+    
+    sel.type("form-myList-1-0", "form-myList-1-0")
+    self.assertEqual("form-myList-0-0", sel.get_value("form-myList-0-0"))
+    self.assertEqual("form-myList-0-1", sel.get_value("form-myList-0-1"))
+    self.assertEqual("form-myList-1-0", sel.get_value("form-myList-1-0"))
+
+    sel.click_at("css=#form-myList-0--field > a", "")
+
+    sel.type("form-myList-0-2", "form-myList-0-2")
+    self.assertEqual("form-myList-0-0", sel.get_value("form-myList-0-0"))
+    self.assertEqual("form-myList-0-1", sel.get_value("form-myList-0-1"))
+    self.assertEqual("form-myList-0-2", sel.get_value("form-myList-0-2"))
+    self.assertEqual("form-myList-1-0", sel.get_value("form-myList-1-0"))
+
+    sel.mouse_down_at("css=#form-myList-0-1--field .remove", "")
+    self.assertEqual("form-myList-0-0", sel.get_value("form-myList-0-0"))
+    self.assertEqual("form-myList-0-2", sel.get_value("form-myList-0-1"))
+    self.assertEqual(False, sel.is_element_present("css=#form-myList-0-2"))
+    self.assertEqual("form-myList-1-0", sel.get_value("form-myList-1-0"))
+
+
+def form_SequenceOfStringsWithSequenceWidgetOptions(request):
+    """
+    A sequence with some defaults
+    """
+    schema = schemaish.Structure()
+    schema.add( 'myList', schemaish.Sequence( schemaish.String() ))
+
+    form = formish.Form(schema, 'form')
+    form['myList'].widget = formish.SequenceDefault(min_start_fields=1,min_empty_start_fields=0, batch_add_count=5)
+    return form
+
   
 def form_SequenceOfUploadStructures(request):
     """
@@ -1640,3 +1780,33 @@ ${form.footer()|n}
 
     """
 
+def form_CustomisedFormLayoutFields(request):
+    """
+    Allow the emission of parts of a form schema using field names.
+    """
+    schema = schemaish.Structure()
+    schema.add( 'firstName', schemaish.String())
+    schema.add( 'surname', schemaish.String())
+    schema.add( 'age', schemaish.Integer())
+    schema.add( 'sex', schemaish.String())
+
+    form = formish.Form(schema, 'form')
+
+    return form
+
+
+def template_CustomisedFormLayoutFields(request):
+    """
+${form.header()|n}
+${form.metadata()|n}
+
+${form.fields(form.fields.keys()[:1])|n}
+<div id="special">
+${form.fields(['surname'])|n}
+</div>
+${form.fields(form.fields.keys()[2:])|n}
+
+${form.actions()|n}
+${form.footer()|n}
+
+    """
