@@ -1,28 +1,23 @@
 function count_previous_fields(o) {
-  var f = o.prevAll('.field').length;
-  var g = o.prevAll('.group').length;
-  if (f > g) {
-    return f;
-  } else {
-    return g;
-  };
-};   
+  return o.prevAll('.field').length;
+}   
    
 function create_addlinks(o) {
   o.find('.adder').each(function() {
     $(this).before('<a class="adderlink">Add</a>');
   });
-};
+}
 
 function get_sequence_numbers(segments, l) {
   var result = Array();
   for(var i=0; i<segments.length; i++) {
     if (isNaN(parseInt(segments[i])) == false) {
+      var segment = segments[i];
       result.push(segment);
     }
   }
   result.push(l);
-  return result
+  return result;
 }
 
 function replace_stars(original, nums, divider) {
@@ -45,11 +40,12 @@ function replace_stars(original, nums, divider) {
 }
 
 function construct(start_segments, n, remainder, divider, strip) {
+  // Takes a set of prefix segments, a number and the remainder plus a flag whether to strip(?)
   var remainder_bits = remainder.split(divider);
-  var remainder = remainder_bits.slice(1,remainder_bits.length-strip).join(divider);
+  remainder = remainder_bits.slice(1,remainder_bits.length-strip).join(divider);
   var result = Array();
   for(var i=0; i<start_segments.length; i++) {
-    segment = start_segments[i];
+    var segment = start_segments[i];
     if (segment != '') {
       result.push(segment);
     }
@@ -60,26 +56,33 @@ function construct(start_segments, n, remainder, divider, strip) {
   } else {
       var out = result.join(divider);
   }
-  return out
-};
+  return out;
+}
 
-function convert_id_to_name(s) {
+function convert_id_to_name(s, formid) {
   var segments=s.split('-');
-  var out = segments.slice(1,segments.length).join('.');
-  return out
+  if (formid == '') {
+    var start_segment = 0;
+  } else {
+    var start_segment = 1;
+  }
+  var out = segments.slice(start_segment,segments.length).join('.');
+  return out;
 }
 
 function renumber_sequences(o) {
-  var n = 0;
+  var formid = $(o).attr('id');
+  var N = {};
   var previous_seqid_prefix = '';
-  o.find('.type-sequence.widget-sequencedefault > div').each( function () {
-    var seqid = $(this).parent().attr('id');
+  o.find('.type-sequence.widget-sequencedefault > .field:not(.type-container)').each( function () {
+    var seqid = $(this).parent().closest('.field').attr('id');
     var seqid_prefix = seqid.substr(0,seqid.length-6);
     if (seqid_prefix != previous_seqid_prefix) {
-      n = 0;
+      N[seqid_prefix] = 0;
     } else {
-      n=n+1;
+      N[seqid_prefix]=N[seqid_prefix]+1;
     }    
+    n = N[seqid_prefix];
     // replace id occurences
     var thisid = $(this).attr('id');
     var newid = seqid_prefix + n + '--field';
@@ -98,43 +101,51 @@ function renumber_sequences(o) {
       $(this).attr('id', construct(seqid_prefix.split('-'),n,name_remainder,'-', 1));
     });
     // replace 'name' occurences
-    $(this).find("[name^='"+convert_id_to_name(seqid_prefix)+"']").each( function () {
+    $(this).find("[name^='"+convert_id_to_name(seqid_prefix, formid)+"']").each( function () {
       var name = $(this).attr('name');
-      var name_remainder = name.substring(convert_id_to_name(seqid_prefix).length, name.length);
-      $(this).attr('name', construct(convert_id_to_name(seqid_prefix).split('.'),n,name_remainder,'.', 1));
+      var name_remainder = name.substring(convert_id_to_name(seqid_prefix, formid).length, name.length);
+      $(this).attr('name', construct(convert_id_to_name(seqid_prefix, formid).split('.'),n,name_remainder,'.', 1));
     });
     previous_seqid_prefix = seqid_prefix;
   });
-  o.find('.type-sequence.widget-sequencedefault > fieldset').each( function () {
-    var seqid = $(this).parent().attr('id');
+  o.find('.type-sequence.widget-sequencedefault > .type-container').each( function () {
+    // Get the id from the parent - hence this will be the prefix once the --field is stripped
+    var seqid = $(this).parent().closest('.field').attr('id');
     var seqid_prefix = seqid.substr(0,seqid.length-6);
+    // We now count the occurences of any item with the same prefix, as long as
+    // we scan through sequentially we should have the correct, incrementing
+    // numbers..
     if (seqid_prefix != previous_seqid_prefix) {
-      n = 0;
+      N[seqid_prefix] = 0;
     } else {
-      n=n+1;
+      N[seqid_prefix]=N[seqid_prefix]+1;
     }    
-    // replace id occurences
+    n = N[seqid_prefix];
+    // Store the old id on the sequence item
     var thisid = $(this).attr('id');
-    //$(this).find('> legend').text(n);
+    // replace id occurences
     var newid = seqid_prefix + n + '--field';
     $(this).attr('id',newid);
+
     // Replace 'for' occurences
     $(this).find("[for^='"+seqid_prefix+"']").each( function () {
       var name = $(this).attr('for');
       var name_remainder = name.substring(seqid_prefix.length, name.length);
       $(this).attr('for', construct(seqid_prefix.split('-'),n,name_remainder,'-', 0));
     });
+
     // Replace 'id' occurences
     $(this).find("[id^='"+seqid_prefix+"']").each( function () {
       var name = $(this).attr('id');
       var name_remainder = name.substring(seqid_prefix.length, name.length);
       $(this).attr('id', construct(seqid_prefix.split('-'),n,name_remainder,'-', 0));
     });
+
     // replace 'name' occurences
-    $(this).find("[name^='"+convert_id_to_name(seqid_prefix)+"']").each( function () {
+    $(this).find("[name^='"+convert_id_to_name(seqid_prefix, formid)+"']").each( function () {
       var name = $(this).attr('name');
-      var name_remainder = name.substring(convert_id_to_name(seqid_prefix).length, name.length);
-      $(this).attr('name', construct(convert_id_to_name(seqid_prefix).split('.'),n,name_remainder,'.',0));
+      var name_remainder = name.substring(convert_id_to_name(seqid_prefix, formid).length, name.length);
+      $(this).attr('name', construct(convert_id_to_name(seqid_prefix, formid).split('.'),n,name_remainder,'.',0));
     });
     previous_seqid_prefix = seqid_prefix;
   });
@@ -142,21 +153,25 @@ function renumber_sequences(o) {
 }
 
 function add_new_item(t,o) {
-    // Get the base64 encoded template
+    var formid = $(o).attr('id');
+    // Get the encoded template
     var code = t.next('.adder').val();
     // Find out how many fields we already have
     var l = count_previous_fields(t.next('.adder'));
     // Get some variable to help with replacing (originalname, originalid, name, id)
     var originalname = t.next('.adder').attr('name');
+    var new_originalname = t.closest('.type-container').attr('id');
+    new_originalname = new_originalname.substr(0,new_originalname.length-6);
+    new_originalname = convert_id_to_name(new_originalname,formid);
     var segments = originalname.split('.');
     // Get the numbers used in the originalname
-    seqnums = get_sequence_numbers(segments, l);
+    var seqnums = get_sequence_numbers(segments, l);
     var originalid = $(o).attr('id')+'-'+segments.join('-');
     segments[ segments.length -1 ] = l;
     var name = segments.join('.');
     var id = $(o).attr('id')+'-'+segments.join('-');
-    // Decode the base64
-    var html = $.base64Decode(code);
+    // Decode the template.
+    var html = decodeURIComponent(code);
     // Add the links and mousedowns to this generated code
     var h = $(html);
     create_addlinks(h);
@@ -167,7 +182,7 @@ function add_new_item(t,o) {
       $(this).attr('name', newname );
     });
     
-    var newid = replace_stars(h.attr('id'),seqnums,'-')
+    var newid = replace_stars(h.attr('id'),seqnums,'-');
 
     h.attr('id',newid);
     h.find("[id]").each( function () {
@@ -190,48 +205,36 @@ function add_new_item(t,o) {
 }
 
 function add_new_items(t,o) {
-   data = t.parent().parent().find('.formish-sequencedata').attr('title').split(',');
+   var data = t.closest('.field').find('.formish-sequencedata').attr('title').split(',');
    for (var i=0; i<data.length; i++) {
-       terms = data[i].split('=');
-       key = terms[0];
+       var terms = data[i].split('=');
+       var key = terms[0];
        if (key == 'batch_add_count') {
-         value = terms[1];
+         var value = terms[1];
          break;
        }
-   }; 
+   } 
    for (var i=0; i<value; i++) {
       add_new_item(t,o);
    }
 }
 
 function add_mousedown_to_addlinks(o) {
-  o.find('.adderlink').mousedown(function() { add_new_items($(this),o)});
-};
+  o.find('.adderlink').mousedown(function() { add_new_items($(this),o);});
+}
 
 function add_remove_buttons(o) {
-  o.find('.addremove .remove').remove() 
-  o.find('.addremove > div > label').each( function() {
+  o.find('.remove').remove();
+  o.find('.seqdelete').each( function() {
     if ($(this).next().text() != 'delete') {
       var x = $('<span class="remove">delete</span>');
       $(this).after(x);
       x.mousedown(function () {
-        $(this).parent().remove();
+        $(this).closest('.field').remove();
         renumber_sequences($('form'));
         add_sortables($('form'));
       });
-    };
-  });
-  o.find('.addremove > fieldset > legend').each( function() {
-    if ($(this).next().text() != 'delete') {
-      var x = $('<span class="remove">delete</span>');
-      $(this).after(x);
-      x.mousedown(function () {
-        $(this).parent().remove();
-        renumber_sequences($('form'));
-        add_sortables($('form'));
-
-      });
-    };
+    }
   });
 }
 
@@ -240,10 +243,9 @@ function order_changed(e,ui) {
 }
 
 function add_sortables(o) {
-  o.find('.sortable > div > label').after('<div class="handle">drag me</div>');
-  o.find('.sortable > fieldset > legend').after('<div class="handle">drag me</div>');
-  o.find('.sortable').sortable({'items':'> .field','stop':order_changed,'handle':'.handle'});
   o.find('.sortable .handle').remove();
+  o.find('.sortable .seqgrab').after('<div class="handle">drag me</div>');
+  o.find('.sortable').sortable({'items':'> .field','stop':order_changed,'handle':'.handle'});
   
 }
 
